<?php

namespace Fittest;

use Fittest\Storage\Storage;
use Fittest\Variation\Variation;

class Fittest 
{
    static private $persistant;
    static private $session;
    
    static public function start(Storage $persistant, Storage $session, $callback = null)
    {
        static::$persistant = $persistant;
        static::$session = $session; 
        static::checkAllGoals();
        if (!is_null($callback))
        {
            $callback();
        }
    }
    
    static public function createExperiment($id)
    {
        $experiment = static::getExperimentById($id);
        if ($experiment instanceof Experiment)
        {
            return $experiment;    
        }
        
        $experiment = Experiment::factory($id);
        static::addExperiment($experiment);
        return $experiment;
    }
    
    static public function addExperiment(Experiment $experiment)
    {
        $list = self::getExperiments();
        $list[$experiment->getId()] = $experiment;
        static::$session->set('fittest_experiments', $list);
    }
    
    static public function getExperiments()
    {
        return static::$session->get('fittest_experiments', array());    
    }
    
    static public function getPersistantExperiments()
    {
        return static::$persistant->get('fittest_experiments', array());    
    }
    
    static public function getExperimentById($id)
    {
        $list = static::getExperiments();
        return (isset($list[$id])) ? $list[$id] : null;    
    }
    
    static public function addVariation(Variation $variation, Experiment $experiment)
    {
        // get ids
        $expid = $experiment->getId();
        $varid = $variation->getId();
        
        // @TODO cache persistant data

        // get numbers
        $list = static::$persistant->get('fittest_experiments', array());
        $list[$expid] = (isset($list[$expid])) ? $list[$expid] : array();
        $list[$expid][$varid] = (isset($list[$expid][$varid])) ? $list[$expid][$varid] : array();
        $chosen = (isset($list[$expid][$varid]['chosen'])) ? $list[$expid][$varid]['chosen'] : 1;
        $awarded = (isset($list[$expid][$varid]['awarded'])) ? $list[$expid][$varid]['awarded'] : 1;
        
        // assign numbers
        $variation->setChosen($chosen);
        $variation->setAwarded($awarded);
        
        $experiment->addVariation($variation);
    }
    
    static public function chooseVariation(Experiment $experiment)
    {
        $variation = $experiment->chooseVariation();
        static::incrementVariation('chosen', $experiment, $variation);
        return $variation;
    }
    
    static public function checkAllGoals()
    {
        $list = static::getExperiments();
        foreach ($list as $experiment)
        {
            foreach($experiment->getGoals() as $goal)
            {
                if ($goal->reached())
                {
                    $variation = $experiment->chooseVariation();
                    static::incrementVariation('awarded', $experiment, $variation);
                }
            }
        }
    }

    static private function incrementVariation($attr, Experiment $experiment, Variation $variation)
    {
        // get ids
        $expid = $experiment->getId();
        $varid = $variation->getId();
        
        // increment once per session
        $sessid = sprintf(
            'fittest_incremented_%s_%s_%s',
            $attr,
            $expid,
            $varid
        );
        if (static::$session->get($sessid, false) === true)
        {
            return;
        }

        // fetch experiments from persistant storage
        $list = static::$persistant->get('fittest_experiments', array());
        
        // construct structure for storage
        $list[$expid] = (isset($list[$expid])) ? $list[$expid] : array();
        $list[$expid][$varid] = (isset($list[$expid][$varid])) ? $list[$expid][$varid] : array();
        $list[$expid][$varid][$attr] = (isset($list[$expid][$varid][$attr])) ? $list[$expid][$varid][$attr] : 1;
        
        // increment the attribute
        $list[$experiment->getId()][$variation->getId()][$attr]++;

        // write the list back to the persistant storage
        static::$persistant->set('fittest_experiments', $list);
        
        // set incremented state in the session
        static::$session->set($sessid, true);
    }
}